/**
 *
 * Copyright (C) 2013 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/
#ifndef BLOBOBJECT_H
#define BLOBOBJECT_H

#include "vector.h"
#include "blob.h"
#include "blobbox.h"
#include "blobtore.h"
#include "blobcylinder.h"
#include "blobcone.h"
#include "box.h"
#include "opengl.h"
#include "matrice.h"

#include <GL/glut.h>
#include <GL/gl.h>										
#include <GL/glu.h>
#include <vector>

#define POLYGONIZE_V 150
#define ERODE_V 100

class blobObject{

 private:
 protected:
  Vector* av;
  Vector* an;
  Vector* ac;
  int* at;
  int avn;
  int atn;

  

  Vector cutpos=Vector(0.0,0.0,0.0);
  Vector position;
  CMatrix rotation;
  Vector color;//obsolète
  Blob* blob;

 public:
  blobObject(const Vector& = Vector(0.0,0.0,0.0), const CMatrix& = CMatrix(), const Vector& = Vector(0.2,0.7,0.6));
  blobObject(const blobObject&);
  virtual ~blobObject();

  virtual void init();
  GLuint GenerateBlob();
  GLuint GenerateCut();

  inline void cutpos_move(int n){
    cutpos+=Vector(0.0,0.0,0.1)*n;
  }

};


class blobObjectTest1:public blobObject
{
 public:
 blobObjectTest1(const Vector& p= Vector(0.0,0.0,0.0), const CMatrix& r = CMatrix(), const Vector& c = Vector(0.2,0.7,0.6)):blobObject(p,r,c){}
 blobObjectTest1(const blobObjectTest1& o):blobObject(o){}
  virtual ~blobObjectTest1(){}
  
  virtual void init();
};

class blobObjectTest2:public blobObject
{
 public:
 blobObjectTest2(const Vector& p= Vector(0.0,0.0,0.0), const CMatrix& r = CMatrix(), const Vector& c = Vector(0.2,0.7,0.6)):blobObject(p,r,c){}
 blobObjectTest2(const blobObjectTest2& o):blobObject(o){}
  virtual ~blobObjectTest2(){}
  
  virtual void init();
};

class blobMaison:public blobObject
{
 protected:
  int typeColor;
  int typeErode;
 public:
 blobMaison(const Vector& p= Vector(0.0,0.0,0.0), const CMatrix& r = CMatrix(), const Vector& c = Vector(0.2,0.7,0.6)):blobObject(p,r,c){}
 blobMaison(const blobMaison& o):blobObject(o){}
  virtual ~blobMaison(){}
  
  virtual void init();
};

class blobMaisonGrande:public blobObject
{
 public:
 blobMaisonGrande(const Vector& p= Vector(0.0,0.0,0.0), const CMatrix& r = CMatrix(), const Vector& c = Vector(0.2,0.7,0.6)):blobObject(p,r,c){}
 blobMaisonGrande(const blobMaison& o):blobObject(o){}
  virtual ~blobMaisonGrande(){}
  
  virtual void init();
};

class blobSchtroumpf:public blobObject
{
 public:
 blobSchtroumpf(const Vector& p= Vector(0.0,0.0,0.0), const CMatrix& r = CMatrix(), const Vector& c = Vector(0.2,0.7,0.6)):blobObject(p,r,c){}
 blobSchtroumpf(const blobMaison& o):blobObject(o){}
  virtual ~blobSchtroumpf(){}
  
  virtual void init();
};

class blobGround:public blobObject
{
 public:
 blobGround(const Vector& p= Vector(0.0,0.0,0.0), const CMatrix& r = CMatrix(), const Vector& c = Vector(0.2,0.7,0.6)):blobObject(p,r,c){}
 blobGround(const blobMaison& o):blobObject(o){}
  virtual ~blobGround(){}
  
  virtual void init();
};

class blobPuit:public blobObject
{
 public:
 blobPuit(const Vector& p= Vector(0.0,0.0,0.0), const CMatrix& r = CMatrix(), const Vector& c = Vector(0.2,0.7,0.6)):blobObject(p,r,c){}
 blobPuit(const blobMaison& o):blobObject(o){}
  virtual ~blobPuit(){}
  
  virtual void init();
};

#endif //BLOBOBJECT_H
